/**
Action Script library. Array class 
Copyright (C) 2005

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "as_array.hh"
 
typedef void (ASArray::*funcPtr)();
static funcPtr callbacks[]= { 
	&ASArray::init,			// 0
	&ASArray::concat,		// 1		
	&ASArray::join,			// 2
	&ASArray::pop,			// 3
	&ASArray::push,			// 4
	&ASArray::reverse,		// 5
	&ASArray::shift,		// 6
	&ASArray::slice,		// 7
	&ASArray::sort,			// 8
	&ASArray::sortOn,		// 9
	&ASArray::splice,		// 10
	&ASArray::AStoString,	// 11
	&ASArray::unshift,		// 12
	// Need to be at the end.
	&ASArray::addProperty,		
	&ASArray::registerClass,	
	&ASArray::AStoString,		
	&ASArray::unwatch,			
	&ASArray::valueOf,			
	&ASArray::watch
};

/**
 * Return a function descriptor
 */
Function *ASArray::getFunctionMember(wstring memberName) {
	Function *func = getBaseFunction(memberName);
	if (memberName == L"Array") 	{func->setFunctionIndex(0); func->setNumParams(30); } // Illimited
	if (memberName == L"concat") 	{func->setFunctionIndex(1); func->setNumParams(10); } // Illimited
	if (memberName == L"join") 		{func->setFunctionIndex(2); func->setNumParams(1); }
	if (memberName == L"pop") 		{func->setFunctionIndex(3); func->setNumParams(10); }
	if (memberName == L"push") 		{func->setFunctionIndex(4); func->setNumParams(10); } // unlimited
	if (memberName == L"reverse") 	{func->setFunctionIndex(5); func->setNumParams(0); }
	if (memberName == L"shift") 	{func->setFunctionIndex(6); func->setNumParams(0); }
	if (memberName == L"slice") 	{func->setFunctionIndex(7); func->setNumParams(2); }
	if (memberName == L"sort") 		{func->setFunctionIndex(8); func->setNumParams(10); }
	if (memberName == L"sortOn") 	{func->setFunctionIndex(9); func->setNumParams(10); }
	if (memberName == L"splice") 	{func->setFunctionIndex(10); func->setNumParams(10); }
	if (memberName == L"toString") 	{func->setFunctionIndex(11); func->setNumParams(0); }
	if (memberName == L"unshift") 	{func->setFunctionIndex(12); func->setNumParams(10); }
	func = finishFunctionSetup(memberName,13,func);
	return func;
}

/**
 * The constructor
 **/
ASArray::ASArray(Script *s, Graphics *g) : ASObject(s,g,L"Array") {
	array = NULL;
}

/**
 * The destructor
 **/
ASArray::~ASArray(){
	if (array != NULL) {
		for (int i=0; i< arrayLen; i++) delete array[i];
		delete []array;
	}
}

/**
 * Call class built in functions
 **/
void ASArray::callFunction(Function *func) {
	numPassedParams = func->getNumPassedParams();
	(this->*callbacks[func->getFunctionIndex()])();
}

/**
 * Creates an Array. Possible constructors are:
 * new Array()
 * new Array(length)
 * new Array(element0, element1, element2,...elementN)
 **/
void ASArray::init() {
	if (numPassedParams == 0) {
		arrayLen = 10;	// Allow some initial space: 10 pointers
		array = new ValueObject*[arrayLen];
		for (int i=0; i< arrayLen; i++) array[i] = NULL;
		maxPosition = 0;
	} else 	if (numPassedParams == 1) {
		arrayLen = int(script->getRegister(1)->toNumeric());
		array = new ValueObject*[arrayLen];
		for (int i=0; i< arrayLen; i++) array[i] = NULL;
		maxPosition = 0;
	} else {
		arrayLen = numPassedParams;
		array = new ValueObject*[arrayLen];
		for (int i=0; i< arrayLen; i++) array[i] = script->getRegister(i+1)->clone();
		maxPosition = arrayLen;
	}
}

/**
 * Allocate array elements (from non action script calls)
 **/
void ASArray::allocate(int nbElements) {
	arrayLen = nbElements;
	array = new ValueObject*[arrayLen];
}

/**
 * We don't use default member behavior for arrays
 * We use our internal variable, not a HashMap
 **/
void ASArray::setMember(wstring memberName, ValueObject *v) {
	//printf("[Array] Set member");
	StringObject *s = new StringObject(memberName);
	int index = int(s->toNumeric());
	delete s;
	if (index < arrayLen) {
		array[index] = v->clone();
	} else {
		ValueObject**array2 = new ValueObject*[index+1];
		for (int i=0; i< arrayLen; i++) array2[i] = array[i];
		arrayLen = index+1;
		delete []array;
		array = array2;
		array[index] = v->clone();
	}
	maxPosition = index;
}

/**
 * We don't use default member behavior for arrays
 * We use our internal variable, not a HashMap
 **/
ValueObject* ASArray::getMember(wstring memberName) {
	if (memberName == L"length") {
		return new IntegerObject(arrayLen);
	} else {
		StringObject *s = new StringObject(memberName);
		int index = int(s->toNumeric());
		delete s;
		if (index < arrayLen) {
			ValueObject *v = array[index];
			return v == NULL ? new StringObject(L"undefined") : v->clone() ;
		} else {
			return new StringObject(L"");
		}
	}
}

/**
 * Concatenates the parameters and returns them as a new array.
 **/
void ASArray::concat() {
	puts("TODO");
	noReturnValue();
}

/**
 * Joins all elements of an array into a string.
 **/
void ASArray::join() {
	puts("TODO");
	noReturnValue();
}

/**
 * Removes the last element of an array and returns its value.
 **/
void ASArray::pop() {
	puts("TODO");
	noReturnValue();
}

/**
 * Adds one or more elements to the end of an array and returns the array’s new length.
 **/
void ASArray::push() {
	for (int i=0; i< numPassedParams;i++) { 
		char tmp[10]; sprintf(tmp,"%d",maxPosition);
		ValueObject *v = script->getRegister(1+i); 
		setMember(stringTowstring(tmp),v);
		if (i != numPassedParams-1) maxPosition++;
	}
	script->push(new IntegerObject(maxPosition));
}

/**
 * Reverses the direction of an array.
 **/
void ASArray::reverse() {
	puts("TODO");
	noReturnValue();
}

/**
 * Removes the first element from an array and returns its value.
 **/
void ASArray::shift() {
	puts("TODO");
	noReturnValue();
}

/**
 * Extracts a section of an array and returns it as a new array.
 **/
void ASArray::slice() {
	puts("TODO");
	noReturnValue();
}

/**
 * Sorts an array in place.
 **/
void ASArray::sort() {
	puts("TODO");
	noReturnValue();
}

/**
 * Sorts an array based on a field in the array.
 **/
void ASArray::sortOn() {
	puts("TODO");
	noReturnValue();
}

/**
 * Adds and removes elements from an array.
 **/
void ASArray::splice() {
	puts("TODO");
	noReturnValue();
}

/**
 * toString: Returns a string value representing the elements in the Array object.
 **/
void ASArray::AStoString() {
	wstring res = L"";
	for (int i=0; i< arrayLen; i++) {
		if (array[i]!=NULL) res += array[i]->toString();
		if (i < arrayLen-1) res += L",";
	}
	script->push(new StringObject(res));
}

/**
 * Adds one or more elements to the beginning of an array and returns the array’s new length.
 **/
void ASArray::unshift() {
	puts("TODO");
	noReturnValue();
}

/**
 * Returns array size as an int
 **/
int ASArray::ASLength() {
	return maxPosition;
}

/**
 * Return the object as s string
 **/
ValueObject*ASArray::getMainMember() {
	AStoString();
	return script->pop();
}
