/**
Action Script library. SharedObject class implementation
Copyright (C) 2005  

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "as_shared_object.hh"

typedef void (ASSharedObject::*funcPtr)();
static funcPtr callbacks[]= { 
	&ASSharedObject::init,			// 0
	&ASSharedObject::clear,			// 1
	&ASSharedObject::flush,			// 2
	&ASSharedObject::getLocal,		// 3
	&ASSharedObject::getSize,		// 4
	// Need to be at the end.
	&ASSharedObject::addProperty,		
	&ASSharedObject::registerClass,	
	&ASSharedObject::AStoString,		
	&ASSharedObject::unwatch,			
	&ASSharedObject::valueOf,			
	&ASSharedObject::watch
};

/**
 * The constructor
 **/
ASSharedObject::ASSharedObject(Script *s, Graphics *g) : ASObject(s,g,L"SharedObject") {
}

/**
 * The destructor
 **/
ASSharedObject::~ASSharedObject() {
}

/**
 * Return a function descriptor
 */
Function *ASSharedObject::getFunctionMember(wstring memberName) {
	Function *func = getBaseFunction(memberName);
	if (memberName == L"SharedObject") 	{func->setFunctionIndex(0); func->setNumParams(0); }
	if (memberName == L"clear") 		{func->setFunctionIndex(1); func->setNumParams(0); }
	if (memberName == L"flush") 		{func->setFunctionIndex(2); func->setNumParams(2); }
	if (memberName == L"getLocal") 		{func->setFunctionIndex(3); func->setNumParams(2); }
	if (memberName == L"getSize") 		{func->setFunctionIndex(4); func->setNumParams(0); }
	finishFunctionSetup(memberName,5,func);
	return func;
}

/**
 * Call class built in functions
 **/
void ASSharedObject::callFunction(Function *func) {
	numPassedParams = func->getNumPassedParams();
	(this->*callbacks[func->getFunctionIndex()])();
}

/**
 *
 **/
void ASSharedObject::init() {
	puts("TODO");
}

/**
 * Purges all of the data from the shared object and deletes the shared object from the disk.
 **/
void ASSharedObject::clear() {
	puts("TODO");
	noReturnValue();
}

/**
 * Immediately writes a locally persistent shared object to a local file.
 **/
void ASSharedObject::flush() {
	puts("TODO");
	noReturnValue();
}

/**
 * Returns a reference to a locally persistent shared object that is available only to the current client.
 **/
void ASSharedObject::getLocal() {
	puts("TODO");
	noReturnValue();
}

/**
 * Gets the current size of the shared object, in bytes.
 **/
void ASSharedObject::getSize() {
	puts("TODO");
	noReturnValue();
}

