AC_DEFUN([AM_LIB_FFMPEG], [
    #Test for ffmpeg
    #See if ffmpeg-config is available...
    AC_MSG_CHECKING([[if ffmpeg-config is present]])
    if ffmpeg-config --version 1>/dev/null 2>/dev/null; then
        AC_MSG_RESULT([[yes]])
        LIBFFMPEG="`ffmpeg-config --plugin-libs avcodec avformat`"
    else
        AC_MSG_RESULT([[no]])
    fi

    # If ffmpeg-config didn't work, see if ffmpeg is registered with pkg-config
    if test "x$LIBFFMPEG" == "x"; then
        PKG_CHECK_MODULES([FFMPEG], [libavcodec libavformat], [LIBFFMPEG="$FFMPEG_LIBS"], [:])
    fi
    
    # Otherwise, try a plain-jane linking and header search.
    if test "x$LIBFFMPEG" == "x"; then
        AC_CHECK_LIB([avformat_pic], [av_register_all],
                     [LIBFFMPEG="-lavformat_pic -lavcodec_pic"],
                     [AC_CHECK_LIB([avformat], [av_register_all],
                                   [LIBFFMPEG="-lavformat -lavcodec"],
                                   [AC_MSG_ERROR([[*** GPLFlash Requires libavcodec and libavformat (ffmpeg)]])],
                                   [-lavcodec]
                                   )],
                     [-lavcodec_pic]
                     )
        
        if test "x$LIBFFMPEG" == "x"; then
            AC_CHECK_HEADER([ffmpeg/avformat.h], [],
                            [AC_MSG_ERROR([[*** ffmpeg header files not found.]])])
        fi
    fi

    AC_SUBST(LIBFFMPEG)
])
AC_DEFUN([AM_LIB_JPEG], [
    #Test for libjpeg
    if test "x$LIBJPEG" == "x"; then
        AC_CHECK_LIB([jpeg], [jpeg_destroy_decompress],
                     [jpeg_ok="yes"],
                     [jpeg_ok="no"])
        if test "$jpeg_ok" == "yes"; then
            AC_CHECK_HEADER([jpeglib.h],
                            [jpeg_ok="yes"],
                            [jpeg_ok="no"])
            if test "$jpeg_ok" == "yes"; then
                LIBJPEG="-ljpeg"
            else
                AC_MSG_ERROR([[*** JPEG header files not found.]])
            fi
        else
            AC_MSG_ERROR([[*** GPLFlash requires libjpeg.]])
        fi
    fi

    AC_SUBST(LIBJPEG)
])
AC_DEFUN([AM_LIB_X], [
    # Test for X11
    X_LIB_PATH_ARGS="-L${x_libraries}"
    X_INCLUDE_PATH_ARGS="-I${x_includes}"
    
    if test "x${x_libraries}" == "x"; then
        X_LIB_PATH_ARGS=""
    fi

    if test "x${x_includes}" == "x"; then
        X_INCLUDES_PATH_ARGS=""
    fi
    AC_CHECK_LIB([X11], [XOpenDisplay],
                 [ac_cv_my_have_x11="yes"
                  X_CFLAGS="${X_INCLUDES_PATH_ARGS}"
                  X_LIBS="-lX11 -lXt -lXext -lXxf86vm ${X_LIB_PATH_ARGS}"
                  AC_DEFINE([USE_X11], [1], [[Define to activate the X11 backend driver]])],
                 [ac_cv_my_have_x11="no"],
                 [[-lXt -lXext -lXxf86vm ${X_LIB_PATH_ARGS}]])
    AC_CHECK_HEADERS([X11/XKBlib.h])
    if test "${ac_cv_my_have_x11}" != "yes" ; then
        AC_MSG_ERROR([[*** cannot find X11 development files]])
    fi

    AC_SUBST(X_CFLAGS)
    AC_SUBST(X_LIBS)
])
AC_DEFUN([AM_LIB_Z], [
    # Test for libz
    if test x"$LIBZ" == "x"; then
        AC_CHECK_LIB([z], [gzsetparams],
                     [z_ok="yes"],
                     [z_ok="no"])
        if test "$z_ok" == "yes"; then
            AC_CHECK_HEADER([zlib.h],
                            [z_ok="yes"],
                            [z_ok="no"])
            if test "$z_ok" == "yes"; then
                LIBZ="-lz"
            else
                AC_MSG_ERROR([[*** LIBZ header files not found.]])
            fi
        else
            AC_MSG_ERROR([[*** GPLFLash requires libz.]])
        fi
    fi

    AC_SUBST(LIBZ)
])
