/**
Macromedia Flash Texture management (Base class - renderer library independant).
Copyright (C) 2004  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _TEXTURE_H_
#define _TEXTURE_H_
#include "common.hh"
#include "swf.hh"
#include "swf_tags.hh"
#include <map>

typedef stdext::hash_map<string, unsigned int, HashString, HashStringCompare> StringBufferHashMap;

class BaseTexture {
public:
	BaseTexture();
	virtual ~BaseTexture();
	void   	registerLinearGradient(int characterId, int fillIndex, FILLSTYLE *f);
	void   	registerRadialGradient(int characterId, int fillIndex, FILLSTYLE *f);
	unsigned int registerCharacterBitmap(int characterId, Image *img);
	unsigned int registerShapeBitmap(int shapeId, Image *img);
	void   	registerExternalBitmap(int shapeId, Image *img);
	int	   	getColorTextureIndex(RGBA *c);
	RGBA   	getColorByIndex(int index);
	RGBA 	getColorByTextureCoords(int texNum, float x, float y);
	unsigned int getGradient(int characterId, int fillIndex);
	unsigned int getExternalTexture(int bitmapId,float &scaleX,float &scaleY,int &dimx, int &dimy);
	unsigned int getCharacterBitmap(int characterId, float &scaleX, float &scaleY);
	unsigned int getShapeBitmap(int shapeId, float &scaleX, float &scaleY);
	bool	bitmapDefined(int bitmapId);
	bool 	characterDefined(int characterId);
	void 	setCharacterDefined(int characterId);
	void	sethasFullTextureOnBoard(bool p) { hasFullTextureOnBoard = p; }

	// These are rendering library specific
	virtual unsigned int allocateTexture(int textureWidth, int textureHeight) = 0;
	virtual unsigned int allocateTexture24(int textureWidth, int textureHeight) = 0;
	virtual int	 registerColor(RGBA *c) =0;
	virtual void video2Texture(int textureId, int dimx, int dimy, char *data) =0;
	virtual void removeTexture(unsigned int textureId) = 0;
	virtual unsigned int getColorTexture() = 0;
	virtual	int	getMaxTextureSize() = 0;
	virtual void storeAlphaBitmap(unsigned int textureID, int xOffset, int yOffset, int destWidth, int destHeight, unsigned char* buffer) = 0;
	// end of rendering library specific
protected:
	ASCIIStringIntHashMap textures, texDimX, texDimY;
	ASCIIStringFloatHashMap texScaleX, texScaleY;
	int 	generatedTexSize;
	StringBufferHashMap ramps;
	IntIntHashMap	colorEntries, invColorEntries;
	int 	nbColorEntries;
	map<unsigned int,RGBA*>	textureCache;
	map<unsigned int,int>	textureCacheDimx,textureCacheDimy;
	bool	hasFullTextureOnBoard;

private:
	// rendering library functions
	virtual unsigned int store1DTextureRepeatLinear(int generatedTexSize, RGBA* textureMemory) = 0;
	virtual unsigned int store2DTextureRepeatLinear( int width, int height, RGBA* textureMemory) = 0;
	// End of rendering library functions
};
#endif
