/**
Macromedia Flash Font management.
Copyright (C) 2005  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _LOCALFONT_H_
#define _LOCALFONT_H_

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <X11/Xft/Xft.h>
#include <vector>
#include "common.hh"
#include "swf.hh"
#include "maths.hh"
#include "btexture.hh"

// TODO: clean that!
#define TTF_PATH "/usr/share/fonts/TTF"

typedef struct {
	unsigned int textureID;
	float x[2],y[2];
	float tcx[2],tcy[2];
	int   advance[2];
} LocalFontEntry;

typedef stdext::hash_map<string, LocalFontEntry, HashString, HashStringCompare> StringLocalFontEntryHashMap;
typedef vector<unsigned int> TextureIDList;
typedef stdext::hash_map<string, TextureIDList*, HashString, HashStringCompare> StringTextureIDListHashMap;
typedef stdext::hash_map<int, TextureIDList*, HashInt, HashIntCompare> IntTextureIDListHashMap;

class LocalFontManager {
public:
	LocalFontManager::LocalFontManager(BaseTexture *texture);
	~LocalFontManager();
	bool loadFontFaces(char *fontName,int fontId,bool isBold, bool isItalic,int codeTableSize, UI16* codeTable);
	bool loadFullFontFaces(char *fontName,int fontId,bool isBold, bool isItalic);
	bool loadGlyphFaces(int fontId,list<UI8*> *bitmaps);
	LocalFontEntry *getChararacterTexture(int fontId, int glyphsId);
	void loadDefaultFont();
	void removeFont(int fontId);
	void setRenderingContext(Display *dpy,Window win) { this->dpy = dpy; this->win = win; }

private:
	FT_Library library;
	StringLocalFontEntryHashMap fontEntries;
	IntTextureIDListHashMap 	globalTextureIds;
	IntTextureIDListHashMap		glyphsTextureIds;
	BaseTexture *texture;
	bool	defaultFontLoaded;
	Display *dpy;
	Window 	win;
	bool	freeType2Support;

	void CalculateTextureSize(int &textureWidth, int &textureHeight, int numGlyphs,int remGlyphs, int glyphWidth,int glyphHeight, int padding);
	XCharStruct *isvalid(XFontStruct * fs, unsigned int which);
	XFontStruct* findXFonts(string originalName,bool isBold,bool isItalic);
	void fillXBitmap(GC gc,int width,int height, int x0, int y0,int c, unsigned char *bitmap);
	bool loadTTFaces(char *fontName,int fontId,int codeTableSize, UI16* codeTable);
	bool loadFullTTFaces(char *fontName,int fontId);
	bool loadXFontFaces(XFontStruct *,int fontId,int codeTableSize, UI16* codeTable);
	bool loadFullXFontFaces(XFontStruct *fs,int fontId);
};
#endif
