/**
Macromedia Flash Processed Object Store.
Copyright (C) 2004  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _OBJECT_STORE_HH
#define _OBJECT_STORE_HH

#include "common.hh"
#include "bgraphics.hh"
#include <stack>
#include <map>
#include "object_types.hh"
#include "swf_function.hh"

enum VariableEntryType {
	NonGraphicObjectType,
	DynamicStringType
};

typedef stdext::hash_map<int, ValueObject *, HashInt,  HashIntCompare> IntValueObjectHashMap;
typedef stdext::hash_map<wstring, IntValueObjectHashMap *, HashUTF8String, HashUTF8StringCompare> StringIntValueObjectHashMapHashMap;
typedef stdext::hash_map<wstring, Function *, HashUTF8String, HashUTF8StringCompare> StringFunctionHashMap;
typedef stdext::hash_map<wstring, ValueObject *, HashUTF8String, HashUTF8StringCompare> StringValueObjectHashMap;
typedef stdext::hash_map<wstring, StringValueObjectHashMap*, HashUTF8String, HashUTF8StringCompare> StringValueObjectHashMapHashMap;
typedef map<int, ScriptObject *> IntScriptObjectMap;
typedef stdext::hash_map<wstring, ScriptObject*, HashUTF8String, HashUTF8StringCompare> StringScriptObjectHashMap;
typedef map<string,ScriptObject *> StringValueObjectMap;

class Graphics;

/*
 * This class is used to store objects that may be get/set in action scripts and in tags
 * 
 **/
class ObjectStore {
public:
	ObjectStore(Graphics *graphics);
	~ObjectStore();
	void		setConstant(int key, ValueObject *p);
	ValueObject *getConstant(int key);
	void 		addFunction(wstring className,wstring functionName, Function *f);
	Function	*getFunction(wstring className,wstring functionName);
	void 		targetPathDown(ScriptObject *);
	void 		targetPathUp();
	void		setActionContext(ScriptObject *s) { currentActionContext = s; }
	void		setPoolActionContext(wstring s);
	wstring		getPoolActionContext() { return poolActionContext; }
	ScriptObject*getActionContext() { return currentActionContext; }
	void 		cleanCurrentLevel();

	void		addLocalVariable(wstring key, ValueObject *v);
	void		setGlobalVariable(wstring key, ValueObject *v);
	void		setSpriteVariable(ScriptObject *so,wstring key, ValueObject *v);
	void 		setVariable(ScriptObject *so,wstring key, ValueObject *v);
	void 		removeLocalVariable(wstring key);
	void 		removeGlobalVariable(wstring key);
	ValueObject*getVariable(ScriptObject *so,wstring key);
	ValueObject*getVariable(wstring absolutePath);
	ValueObject*getGlobalVariable(wstring key);
	ValueObject*getSpriteVariable(ScriptObject *so, wstring key);
	RegisterSet *getGlobalRegisters() { return globalRegisters; }
	ObjectMembers *getGlobalMembers() { return &globalMembers; }
	void 		removeSpriteVariable(ScriptObject *so,wstring key);
	void		registerNewObject(ScriptObject *p) { allocatedObjects[p] =p; }
	void		unRegisterNewObject(ScriptObject *p);
	void		removeObject(ValueObject *v);
	wstring		getCurrentTargetPath() { return currentTargetPath; }
	void		setCurrentTargetPath(wstring p) { currentTargetPath = p; }
	ScriptObject*getMovieByNameOrPath(wstring namePath);

	void		addClonedObject(int characterId, ScriptObject *);
	void		removeClonedObject(int characterId, ScriptObject *);
	list<ScriptObject*>getClonedObjects(int characterId);

	// Local implementation of export asset
	void		addExportedAsset(wstring name, int characterId);
	int			getExportedAssetCharacterId(wstring name) { return exportedAssets[name]; }

	void		setObjectAtDepth(int depth, ScriptObject *object) { objectAtDepth[depth] = object; }
	void		removeObjectAtDepth(int depth) { objectAtDepth.erase(depth); }
	ScriptObject *getObjectAtDepth(int depth) { return objectAtDepth[depth]; }
	IntScriptObjectMap *getObjectByDepth() { return &objectAtDepth; }
	void		clear() { objectAtDepth.clear(); }
	void		resetProgram();
	void		addDeletedObject(ScriptObject *parent,int depth,int characterId, ScriptObject *so); 
	ScriptObject *getDeletedObject(ScriptObject *parent,int depth,int characterId);
	void		destroyObject(ScriptObject*);
	wstring		buildKey(ValueObject *so) { char tmp[100];	sprintf(tmp,"%p",so); return stringTowstring(tmp); }
	void		copyCurrentPoolContext(wstring key);
	void		setLinkage(wstring s, ScriptObject *so);
	wstring		getLinkage(wstring s);
	ScriptObject*getActionScriptObjectClass(ScriptObject *so);
	ScriptObject*getActionScriptObjectClass(wstring s);
	void		setRootReference(ScriptObject *p) { global = p; }
	void 		setHasLocalVariableScope(bool p) { hasLocalVariableScope = p; }

private:
	Graphics 					*graphics;
	StringIntValueObjectHashMapHashMap constantPools;
	StringFunctionHashMap 		functions;
	wstring						currentTargetPath;
	stack<wstring>				currentTargetPathStack;
	StringValueObjectHashMapHashMap	variables;
	RegisterSet*				globalRegisters;
	ScriptObject*				currentActionContext;
	wstring						poolActionContext;
	ObjectMembers				globalMembers; // declaration members, not members of a class instance
	map<ScriptObject*,ScriptObject*>allocatedObjects;	// Defines objects that have been allocated.
	multimap<long,ScriptObject*>clonedObjects;
	map<wstring,int>			exportedAssets;
	map<int,wstring>			invExportedAssets;
	IntScriptObjectMap 			objectAtDepth;		// Rendered object (movie, text) name at given depth
	StringValueObjectMap		rubishStore;		// Deleted objects that may be reused.
	StringScriptObjectHashMap	linkages;			// as required by Object.registerClass
	ScriptObject				*global;
	bool						hasLocalVariableScope;	// for onClipEvent special variable scope handling

	wstring		findVariableContext(ScriptObject *so, wstring key);
	ValueObject* getObjectByDotPath(ScriptObject *root,wstring key);
};
#endif
