/**
Macromedia Flash Script execution engine.
Copyright (C) 2004  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _SWF_STACK_HH_
#define _SWF_STACK_HH_
#include "common.hh"
#include "swf.hh"
#include <iostream>
#include <stack>
#include "object_store.hh"
#include <sys/time.h>

class ASLinker;
class SwfEngine;
class EventManager;

/**
 * The script class
 **/
class Script {
public:
	Script(ObjectStore *p, Graphics *g,SwfEngine *e);
	~Script();

	// DEBUG
	void dumpStack();
	// SWF3 actions
	void doActionGotoFrame(UI8 **p);
	void doActionGetURL(UI8 **p);
	void doActionNextFrame();
	void doActionPrevFrame();
	void doActionPlay();
	void doActionStop();
	void doActionToggleQuality();
	void doActionStopSounds();
	void doActionWaitForFrame(UI8 **p);
	void doActionSetTarget(UI8 **p);
	void doActionGoToLabel(UI8 **p);

	// SWF4 actions
	bool doActionPush(UI8 **,int length); 
	void doActionPop(UI8 **p);
	void doActionAdd();
	void doActionSubstract();
	void doActionMultiply();
	void doActionDivide();
	void doActionEqual();
	void doActionLess();
	void doActionAnd();
	void doActionOr();
	void doActionNot();
	void doActionStringEquals();
	void doActionStringLength();
	void doActionStringAdd();
	void doActionStringExtract();
	void doActionStringLess();
	void doActionMBStringLength();
	void doActionMBStringExtract();
	void doActionToInteger();
	void doActionCharToAscii();
	void doActionAsciiToChar();
	void doActionMBCharToAscii();
	void doActionMBAsciiToChar();
	void doActionGetVariable();
	void doActionSetVariable();
	void doActionIf(UI8 **p);
	void doActionJump(UI8 **p);
	void doActionGetTime();
	void doActionRandomNumber();
	void doActionGetProperty();
	void doActionSetProperty();
	void doActionCall();
	void doActionGetURL2(UI8 **p);
	void doActionGotoFrame2(UI8 **p);
	void doActionSetTarget2();
	void doActionCloneSprite();
	void doActionRemoveSprite();
	void doActionStartDrag();
	void doActionEndDrag();
	void doActionWaitForFrame2(UI8 **p);
	void doActionTrace();

	// SWF5 actions
	void doActionConstantPool(UI8 **p);
	void doActionDefineFunction(UI8 **p);
	void doActionStoreRegister(UI8 **p);
	void doActionCallMethod(UI8 **p);
	void doActionDefineLocal();
	void doActionDefineLocal2();
	void doActionNewObject(UI8 **p);
	void doActionAdd2();
	void doActionLess2();
	void doActionEquals2();
	void doActionIncrement();
	void doActionDecrement();
	void doActionCallFunction(UI8 **p);
	void doActionReturn(UI8 **p);
	void doActionGetMember();
	void doActionSetMember();
	void doActionBitAnd();
	void doActionModulo();
	void doActionBitLShift();
	void doActionBitOr();
	void doActionBitRShift();
	void doActionBitURShift();
	void doActionBitXor();
	void doActionPushDuplicate();
	void doActionInitArray();
	void doActionInitObject();
	void doActionDelete();
	void doActionDelete2();
	void doActionEnumerate();
	void doActionNewMethod(UI8 **p);
	void doActionTargetPath();
	void doActionWith(UI8 **p);
	void doActionToNumber();
	void doActionToString();
	void doActionTypeOf();
	void doActionStackSwap();

	// SWF6 actions
	void doActionGreater();
	void doDoInitAction();
	void doActionInstanceOf();
	void doActionEnumerate2();
	void doActionStrictEquals();
	void doActionStringGreater();
	
	// SWF7 actions
	void doActionDefineFunction2(UI8 **p);
	void doActionExtends();
	void doActionCastOp();
	void doActionImplementsOp();
	void doActionTry();
	void doActionThrow();

	// Others
	void 		pushExtraValue() { execStack.push(new ValueObject()); }
	bool 		checkImplicitReturn(UI8 **p) { return *p == implicitReturnsTarget.top();}
	void 		popImplicitReturn() { implicitReturnsTarget.pop(); }
	UI8*		getImplicitReturn() { return implicitReturnsTarget.top(); }
	UI8* 		popImplicitReturnAddr() { UI8*r = implicitReturns.top(); implicitReturns.pop(); return r; }
	ObjectStore *getObjectStore() { return objectStore; }
	ValueObject *pop();
	ValueObject *top() { return execStack.top(); }
	void		push(ValueObject *p) { execStack.push(p); }
	ValueObject *getRegister(int index) { return currentRegisterSet->getRegisterNoClone(index); }
	Graphics	*getGraphics() { return graphics; }
	SwfEngine	*getEngine() { return engine; }
	void		setLabel(wstring labelName, int frame) { labels[labelName] = frame; }
	ScriptObject*getCurrentContext() { return currentContext; }
	void		removeObject(ValueObject *v);
	EventManager*getEventManager();
	void		prepareExecEnv(ScriptObject *sc);
	void		prepareExecEnv(ScriptObject *sc, wstring key);
	void		leaveExecEnv();
	bool		getHasTargetDefined() { return hasTargetDefined; }
	ScriptObject*getSingleton(wstring className);
	void		setCurrentContext(ScriptObject *p) { currentContext = p; }
	ASLinker	*getASLinker() { return asLinker; }

private:
	stack<ValueObject*> execStack;
	ObjectStore 		*objectStore;
	Graphics			*graphics;
	SwfEngine			*engine;
	ScriptObject		*currentContext;
	RegisterSet			*currentRegisterSet;
	stack<ScriptObject*>actionContextStack;
	stack<wstring>		poolActionContextStack;
	stack<RegisterSet*>	registerBankStack;
	stack<UI8*> 		implicitReturnsTarget; // a constructor does not end with a return statement
	stack<UI8*> 		implicitReturns; 
	ASLinker 			*asLinker;
	struct timeval 		tStart;
	ScriptObject		*rootScript;
	StringIntHashMap	labels;
	int 				swfVersion;
	bool				hasTargetDefined;

	void  		setTop(ValueObject *p) { execStack.top() = p; }
	void		setRegister(int index, ValueObject *p) { currentRegisterSet->setRegister(index,p); }
	ValueObject **getRegisterRef(int index) { return currentRegisterSet->getRegisterRef(index); }
	void		setGlobalRegisterBank() { currentRegisterSet = objectStore->getGlobalRegisters(); }
	void		setRegisterBank(RegisterSet *s);
	void		_doCall(Function* func, int numArgs,ValueObject** args,ScriptObject *scriptObject);
	Function 	*_callBuiltInFunction(ScriptObject **,wstring methodName);
	int			getLabel(wstring labelName);
	void		actionContextPush(ScriptObject *newObject,wstring movieName);
	void		actionContextPop();
};
#endif

