/**
gplflash Player entry point.
Copyright (C) 2005

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "../lib/common.hh"
#include "../lib/swf.hh"
#include "../lib/swf_decoder.hh"
#include "../lib/swf_engine.hh"
#include "../lib/localFonts.hh"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <GL/glx.h>
#include <X11/cursorfont.h>
#include <sys/stat.h>

#define SCALE (1.0/20.0)
#define VALGRIND false

static int attributeList_noFSAA[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, None };
static int attributeList_FSAA[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, GLX_SAMPLE_BUFFERS_ARB, 1,GLX_SAMPLES_ARB, 1, None };

static int currentX11Cursor;
int 	newX11Cursor;
bool debug = true;
bool actionScriptCriticalFailure = false;

/**
 * Opens the  file, get its property and call the decoder
 **/
int openDecodeStream(char *fileName, Swf_decoder * decoder) {
	struct stat buf;
	lstat(fileName, &buf);
	FILE *f = fopen(fileName,"rb");
	if (f == NULL) {
		wcout << "FILE NOT FOUND!";
		return false;
	}
	char *buffer = (char*) malloc(buf.st_size);
	fread(buffer,buf.st_size,1,f);
	fclose(f);
	int status = decoder->readStream(buffer,buf.st_size);
	free(buffer);
	return status;
}

/**
 * Wait for an X event, but do not block for more than millis milliseconds.
 * Returns true, if got an event. It will be available through event_return.
 *         false, if timeout occured before any events
 **/
bool XNextEventTimed(Display* dsp, XEvent* event_return, long millis) {
	if (millis < 0)
		millis = 0;

	if (XPending(dsp) == 0) {
		int fd = ConnectionNumber(dsp);
		struct timeval tv;
		fd_set readset;

		tv.tv_sec = millis/1000;
		tv.tv_usec = 1000*(millis % 1000);
		FD_ZERO(&readset);
		FD_SET(fd, &readset);
		if (select(fd+1, &readset, NULL, NULL, &tv) == 0) {
			return false;
		} else {
			XNextEvent(dsp, event_return);
			return true;
		}
	} else {
		XNextEvent(dsp, event_return);
		return true;
	}
}

/**
 * Open X11 rendering window & process event loop
 **/
void render(Swf_decoder * decoder,bool normalSpeed) {
	Display *dpy;
	XVisualInfo *vi;
	Colormap cmap;
	XSetWindowAttributes swa;
	Window win;
	GLXContext cx;
	XEvent event;
	long timeout, currentTime, lastUpdate;

	dpy = XOpenDisplay(0);
	if ( ! dpy ) CRITICAL_ERROR(L"Can't open display");

	vi = glXChooseVisual(dpy, DefaultScreen(dpy), attributeList_FSAA);
	if (vi == NULL) vi = glXChooseVisual(dpy, DefaultScreen(dpy), attributeList_noFSAA);
	cx = glXCreateContext(dpy, vi, 0, GL_TRUE);
	cmap = XCreateColormap(dpy, RootWindow(dpy, vi->screen), vi->visual, AllocNone);
	swa.colormap = cmap;
	swa.border_pixel = 0;
	swa.event_mask = StructureNotifyMask | ExposureMask | KeyPressMask | KeyReleaseMask| ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
	HEADER *header = decoder->getHeader();
	float dimx = (header->frameSize.xmax-header->frameSize.xmin);
	float dimy = (header->frameSize.ymax-header->frameSize.ymin);
	win = XCreateWindow(dpy, RootWindow(dpy, vi->screen), 0, 0,
				 int(dimx*SCALE), int(dimy*SCALE),
                 0, vi->depth, InputOutput, vi->visual,
                 CWBorderPixel|CWColormap|CWEventMask, &swa);
    XMapWindow(dpy, win);
	currentX11Cursor = XC_left_ptr;

    glXMakeCurrent(dpy, win, cx);
	glViewport(0, 0,int(dimx*SCALE),int(dimy*SCALE));
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,dimx,dimy,0,1,-1);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	SwfEngine *engine = new SwfEngine(decoder);

	int frame = 1;
	engine->next();
	bool redraw = true;
	int timeBetweenFrames = int(1000.0/header->frameRate);

	struct timeval now;
	gettimeofday(&now, NULL);
	lastUpdate = now.tv_sec*1000 + now.tv_usec/1000;

#if VALGRIND
	for (int i=0; i< 5; i++) {
#else
    while(1) {
#endif
		gettimeofday(&now, NULL);
		currentTime = now.tv_sec*1000 + now.tv_usec/1000;
		timeout = timeBetweenFrames - (currentTime-lastUpdate);

		bool eventFlag = XNextEventTimed(dpy, &event, timeout);
		if (eventFlag) {
			// got event
			switch (event.type) { 
	        	case KeyPress: {
					KeySym key;
    	            XLookupString((XKeyEvent *)&event, NULL, 0, &key, NULL); 
					char code[5];
					XLookupString((XKeyEvent *)&event, code, 5, NULL, NULL); 
					//if (key == 65361) { if (frame>0) frame--; engine->goToMainTrackFrame(frame); }
					//else if (key == 65363) { frame++; engine->next(); }
					if (key == 65307) {	delete engine; delete decoder; exit(0); }
					else engine->keyboardCallback(key,code);
					redraw = true;
					break;
				}
				case KeyRelease: {
					engine->setKeyPressed(false);
					break;
				}
				case ButtonPress: {
					int x = event.xbutton.x;
					int y = event.xbutton.y;
					// convert to twips
					x = int (x / SCALE);
					y = int (y / SCALE);
					redraw = engine->mouseClickCallback(x,y,MouseActionPress);
					break;
				}
			    case ButtonRelease: {
					int x = event.xbutton.x;
					int y = event.xbutton.y;
					// convert to twips
					x = int (x / SCALE);
					y = int (y / SCALE);
					redraw = engine->mouseClickCallback(x,y,MouseActionRelease);
					break;
				}
				case MotionNotify: {
					int x = event.xbutton.x;
					int y = event.xbutton.y;
					// convert to twips
					x = int (x / SCALE);
					y = int (y / SCALE);
					redraw = engine->mouseClickCallback(x,y,MouseActionOver);
					break;
				}
				case Expose: {
					redraw = true;
					break;
				}
			}
		}
		if (!eventFlag || timeout <= 0) {
			// time to show the next frame
			if (normalSpeed) {
				frame++; 
				engine->next(); 
				redraw = true;
				gettimeofday(&now, NULL);
				lastUpdate = now.tv_sec*1000 + now.tv_usec/1000;
			}
		}

		if (redraw) {
			engine->display();
			glXSwapBuffers(dpy, win);
			redraw = false;
		}
		if (currentX11Cursor != newX11Cursor) { 		// X11 Cursor change detected
			Cursor c = XCreateFontCursor (dpy, newX11Cursor);
			XDefineCursor (dpy, win, c);
			currentX11Cursor = newX11Cursor;
		}
	}
	delete engine;
	delete decoder;
	free(vi);
}

/**
 * If a critical erro occurs in player display it.
 * if it occurs in the plugin, ignore it
 **/
void CRITICAL_ERROR(wstring error) {
	wcout << L"\nPROGRAM STOPPED!\nThe following critical error occurs: " << error << L"\n";
	exit(0);
}

/**
 * Entry point
 **/
int main(int argc, char ** argv) {
	bool normalSpeed = false;
	char *fileName;
	if (argc > 1) {
		fileName = argv[1];
		if (argc > 2) normalSpeed = true;
	} else {
		cout << "Usage: flashplayer fileName [autoPlay]\n";
		exit(0);
	}
	
	Swf_decoder * decoder = new Swf_decoder();
	int ret = openDecodeStream(fileName, decoder);
	if (ret == 0) {
		cout << "Decoding fails!\n. Bye.";
	}
	render(decoder,normalSpeed);
	return 0;
}
