/**
Plugin interface. Mozilla / Konqueror.
Copyright (C) 2005  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "../../lib/common.hh"
#include "../../lib/swf.hh"
#include "../../lib/swf_decoder.hh"
#include "../../lib/swf_engine.hh"
#include "../../lib/localFonts.hh"
#include "flash_interface.hh"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>
#include <pthread.h>

static pthread_mutex_t  synchro;
static int attributeList_noFSAA[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, None };
static int attributeList_FSAA[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, GLX_SAMPLE_BUFFERS_ARB, 1,GLX_SAMPLES_ARB, 1, None };
int    newX11Cursor, currentX11Cursor;
bool debug = true;
bool actionScriptCriticalFailure = false;

extern "C" {

FlashHandle flashNew() {
	PluginInterface *pi = new PluginInterface();
	return pi;
}

int addBytesToDecoder(void *data) {
	tNSBufferDescriptor *buffer = (tNSBufferDescriptor*) data;
	PluginInterface *pi = (PluginInterface *)buffer->flashHandle;
	long retval = pi->addBytesToDecoder(buffer->data, buffer->offset,buffer->len);
	return retval;
}

void setWindow(FlashHandle flashHandle,Window win,Display* dpy) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	pi->setWindow(win,dpy);
}

void flashTest(FlashHandle flashHandle) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	pi->test();
}

int  getTimerInterval(FlashHandle flashHandle) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	return pi->getTimerInterval();
}

void flashRender(void *data) {
	PluginInterface *pi = (PluginInterface *)data;
	pi->flashRender();
}

void closeFlash(FlashHandle flashHandle) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	delete pi;
}

int	mouseClickCallback(FlashHandle flashHandle,int x, int y,int mouseAction) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	return pi->mouseClickCallback(x,y,mouseAction);
}

void keyboardCallback(FlashHandle flashHandle,KeySym key,char*code) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	pi->keyboardCallback(key,code);
}

void keyboardReleased(FlashHandle flashHandle) {
	PluginInterface *pi = (PluginInterface *)flashHandle;
	pi->setKeyPressed(false);
}
}; // end of extern C


/**
 * Initialize an instance plugin
 **/
PluginInterface::PluginInterface() {
	//printf("PluginInterface::PluginInterface(): %x\n",this);
	decoder = new Swf_decoder();
	internalBuffer = NULL;
	internalBufferSize = 0; 
	engine = NULL;
	renderInProgress = false;
}
 
/** 
 * The destructor
 **/
PluginInterface::~PluginInterface() {
	if (engine != NULL)  delete engine;
	if (decoder != NULL) delete decoder;
}

/**
 * Add bytes to the flash 
 **/
int PluginInterface::addBytesToDecoder(const char *data,int offset, int len) {
	internalBuffer = (char*) realloc(internalBuffer,internalBufferSize+len);
	memcpy(internalBuffer+internalBufferSize,data,len);
	internalBufferSize += len;

	bool status = decoder->readStream(internalBuffer,internalBufferSize); 
	if (status) {							// Decode was, bytes used.
		if (internalBuffer != NULL) {
			free(internalBuffer);
			internalBufferSize = 0;
			internalBuffer= NULL;
		}
	} 
		
	if (status) {
		timeBetweenFrames = int(1000.0/decoder->getHeader()->frameRate);
		engine = new SwfEngine(decoder);
		// Set transformation MATRIX
		header = decoder->getHeader();
		float fdimx = (header->frameSize.xmax-header->frameSize.xmin);
		float fdimy = (header->frameSize.ymax-header->frameSize.ymin);
		glXMakeCurrent(dpy,win,context);
		glViewport(0, 0,int(dimx),int(dimy));
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0,int(fdimx),int(fdimy),0,1,-1);
	}
	return status ? 1 : 0;
}

/**
 * Configure Flash script drawing area
 *
 * Since all OpenGl context runs in the same window, we need to change
 **/
/**
 * Prepare OpenGl drawing area
 **/
void PluginInterface::setWindow(Window win,Display* dpy){
	this->win = win;
	this->dpy = dpy;
	XWindowAttributes wattr;
    XGetWindowAttributes(dpy, win, &wattr);
	dimx = wattr.width;
	dimy = wattr.height;
	initGraphic();
}

/**
 * Init OpenGL rendering
 * First try to use FSAA - FUll Screen Anti Aliasing
 * If the hardware does not support it, go back to non aliased mode.
 **/
void PluginInterface::initGraphic() {
	XVisualInfo *vi = glXChooseVisual(dpy, DefaultScreen(dpy), attributeList_FSAA);
	if (vi == NULL) vi = glXChooseVisual(dpy, DefaultScreen(dpy), attributeList_noFSAA);
	context = glXCreateContext(dpy, vi, 0, GL_TRUE);
    glXMakeCurrent(dpy, win, context);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
}

/** 
 * Call the flash engine rendering - Reentrant function.
 **/
void PluginInterface::flashRender() {
	if (engine == NULL) return;			// Nothing ready for rendering.
	engine->getFontManager()->setRenderingContext(dpy,win);
	if (renderInProgress) return;		// OVERRUN!!!!
	renderInProgress = true;
	//pthread_mutex_lock(&synchro);
	//XLockDisplay(dpy);
	//XFlush(dpy);
	glXMakeCurrent(dpy,win,context);
	try {
		engine->next();
		engine->display();	// No Overrun. We have time to display something.
	}  catch ( ... ) {
		printf("Flash error .......\n");
	}
	glXSwapBuffers(dpy, win);
	//usleep(100);
    //glXMakeCurrent(dpy, None, NULL);
	//XUnlockDisplay(dpy);
	//pthread_mutex_unlock(&synchro);
	renderInProgress = false;
}


/**
 * Mouse call back handling
 * Transform mouse coordinates back to flash coordinates.
 **/
int	PluginInterface::mouseClickCallback(int x, int y,int mouseAction) {
	if (engine == NULL) return 0;	// Not loaded yet
	x = (x * (header->frameSize.xmax-header->frameSize.xmin)) / dimx;
	y = (y * (header->frameSize.ymax-header->frameSize.ymin)) / dimy;
	engine->mouseClickCallback(x,y,mouseAction);
	if (currentX11Cursor != newX11Cursor) { 		// X11 Cursor change detected
		Cursor c = XCreateFontCursor (dpy, newX11Cursor);
		XDefineCursor (dpy, win, c);
		currentX11Cursor = newX11Cursor;
	}
	return 0;
}

/**
 * Keyboard callback
 **/
void PluginInterface::keyboardCallback(int key, char *code) {
	engine->keyboardCallback(key,code);
}

void PluginInterface::test(){
	bool status = glXMakeCurrent(dpy,win,context);
	if (!status) exit(0);
	glClearColor(0.0,1,0,1);
	glClear(GL_COLOR_BUFFER_BIT); 
	glFlush();	
	glXSwapBuffers(dpy, win);
printf("ERROR: %d\n",glGetError());
}

void PluginInterface::setKeyPressed(bool p) {
	engine->setKeyPressed(p);
}

/**
 * If a critical erro occurs in player display it.
 * if it occurs in the plugin, ignore it
 **/
void CRITICAL_ERROR(wstring error) {
	wcout << L"\nACTION SCRIPT STOPPED!\nThe following critical error occurs: " << error << L"\n";
	actionScriptCriticalFailure=true;
	// continue execution?
}

